/*******************************************************************************
 * I9 TECHNOLOGIES CONFIDENTIAL
 * __________________
 * Copyright © 2013 i9 Technologies 
 * All Rights Reserved.
 * 
 * NOTICE:  All information contained herein is, and remains the property 
 * of i9 Technologies and its suppliers, if any.  The intellectual and technical 
 * concepts contained herein are proprietary to i9 Technologies and its 
 * suppliers and may be covered by U.S. and Foreign Patents, patents in 
 * process, and are protected by trade secret or copyright law. Dissemination 
 * of this information or reproduction of this material is strictly forbidden 
 * unless prior written permission is obtained from i9 Technologies.
 ******************************************************************************/
package com.xmlnamespace.panel.vaadin.shared.ui;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

/**
 * Base class for items in a UserComponent
 */
public class UserComponentItem implements Serializable {

    /** The id. */
    private double id;
    
    /** The styles. */
    private List<String> styles = new ArrayList<String>();
    
    /** The tool tip. */
    private String toolTip;
    
    /** The change listeners. */
    private List<UserComponentItemChangeListener> changeListeners = new ArrayList<UserComponentItemChangeListener>();

    /** The Constant serialVersionUID. */
    private static final long serialVersionUID = 1L;

    /**
     * Instantiates a new user component item.
     */
    public UserComponentItem() {
        id = Math.random();
    }

    /**
     * Gets the id.
     *
     * @return the id
     */
    public double getId() {
        return id;
    }

    /**
     * Sets the id.
     *
     * @param id the new id
     */
    public void setId(double id) {
        this.id = id;
    }

    /**
     * Gets the tool tip.
     *
     * @return the tool tip
     */
    public String getToolTip() {
        return toolTip;
    }

    /**
     * Sets the tool tip.
     *
     * @param toolTip the new tool tip
     */
    public void setToolTip(String toolTip) {
        this.toolTip = toolTip;
        fireChangeListeners();
    }

    /**
     * Adds a style.
     *
     * @param style the style
     */
    public void addStyle(String style) {
        if (!styles.contains(style)) {
            styles.add(style);
            fireChangeListeners();
        }
    }

    /**
     * Removes a style.
     *
     * @param style the style
     */
    public void removeStyle(String style) {
        styles.remove(style);
        fireChangeListeners();
    }
    
    /**
     * Sets the styles.
     * 
     * For internal use only.
     * Will not update the item styles when called.
     *
     * @param styles the styles
     * @return the list
     */
    public void setStyles(List<String> styles) {
        this.styles = styles;
    }

    /**
     * Gets the styles.
     *
     * @return the styles
     */
    public List<String> getStyles() {
        return styles;
    }

    /**
     * Adds a change listener.
     *
     * @param listener the listener
     */
    public void addChangeListener(UserComponentItemChangeListener listener) {
        changeListeners.add(listener);
    }

    /**
     * Remove a change listener.
     *
     * @param listener the listener
     */
    public void removeChangeListener(UserComponentItemChangeListener listener) {
        changeListeners.remove(listener);
    }

    /**
     * Gets the change listeners.
     *
     * @return the change listeners
     */
    public List<UserComponentItemChangeListener> getChangeListeners() {
        return changeListeners;
    }

    /**
     * Fire change listeners.
     */
    protected void fireChangeListeners() {
        for (UserComponentItemChangeListener listener : changeListeners) {
            listener.userComponentItemChanged(new UserComponentItemChangedEvent(
                    this));
        }
    }

    /**
     * The listener interface for receiving userComponentItemChange events.
     * The class that is interested in processing a userComponentItemChange
     * event implements this interface, and the object created
     * with that class is registered with a component using the
     * component's <code>addUserComponentItemChangeListener<code> method. When
     * the userComponentItemChange event occurs, that object's appropriate
     * method is invoked.
     *
     * @see UserComponentItemChangeEvent
     */
    public static interface UserComponentItemChangeListener {

        /**
         * User component item changed.
         *
         * @param event the event
         */
        public void userComponentItemChanged(UserComponentItemChangedEvent event);

    }

    /**
     * Event fired when the user component item changes.
     * 
     * This is used to inform the UserComponent that one
     * of its items has changed and an event needs to be sent
     * to the client or the state needs to be marked as dirty.
     */
    public static class UserComponentItemChangedEvent {

        /** The item. */
        private UserComponentItem item;

        /**
         * Instantiates a new user component item changed event.
         *
         * @param item the item that has changed
         */
        public UserComponentItemChangedEvent(UserComponentItem item) {
            this.item = item;
        }

        /**
         * Gets the item that has changed.
         *
         * @return the item that has changed
         */
        public UserComponentItem getItem() {
            return item;
        }

        /**
         * Sets the item that has changed.
         *
         * @param item the item that has changed
         */
        public void setItem(UserComponentItem item) {
            this.item = item;
        }

    }

}
